/*
 * LEAF Visual Novel System For X
 * (c) Copyright 1999,2000 Go Watanabe mailto:go@denpa.org
 * All rights reserverd.
 *
 * ORIGINAL LVNS (c) Copyright 1996-1999 LEAF/AQUAPLUS Inc.
 *
 * $Id: Lvns.h,v 1.1 2002/07/29 05:24:38 go Exp $
 *
 */

#ifndef __Lvns_h
#define __Lvns_h

#ifdef unix
#include <sys/types.h>
#else
typedef unsigned char  u_char;
typedef unsigned short u_short;
typedef unsigned long  u_long;
#endif

#include <setjmp.h>

#ifdef DEBUG
#define dprintf(a) fprintf a
#else
#define dprintf(a)
#endif

#ifndef Bool
#define Bool int
#endif

#ifndef True
#define True 1
#endif

#ifndef False
#define False 0
#endif

#include "leafpack.h"   /* Leaf Pack File   */
#include "Sound.h"
#include "Music.h"
#ifndef USE_MGL
#include "lvnsimage.h"  /* Leaf Image Data  */
#else
#include "mgl2.h"
#include "mglcol.h"
#include "mgImage.h"
#ifdef FONT_PLUS
#include "mgFont.h"
#endif /* FONT_PLUS */
#endif

#ifndef USE_MGL
#define VERSION "LEAF Visual Novel System for X\n"\
                " ==== XLVNS 1-2-3 Ver 3.0beta ====\n"\
                "(c) Copyright 1999,2000 Go Watanabe\n"\
                "Original LVNS (c)LEAF/AQUAPLUS\n"
#else
#define VERSION "LEAF Visual Novel System for MGL2\n"\
                " ==== mglvns 1-2-3 Ver 1.0beta ====\n"\
                "(c) Copyright 1999-2001 Go Watanabe\n"\
                "(c) Copyright 2001 TF\n"\
                "Original LVNS (c)LEAF/AQUAPLUS\n"
#endif

#define INTERVAL 60
#define NOCHARACTER 255

/* EUC ɤѥå JIS ɤء */
#define EucToJisPack(code)  ((((code>>8)&0x7f)-33)*94 + ((code&0x7f)-33))

#define LVNS     lvns->system_state
#define COLORS  (lvns->system_state->colors)
#define WIDTH	(lvns->system_state->width)
#define HEIGHT	(lvns->system_state->height)
#ifndef USE_MGL
#define WIDTH2       640
#define CHARSIZE     24
#define CHARDRAWSIZE 26
#define EDYOFF       30
#define XPOS(x,y)    ((x) * 24 + lvns->tvram[lvns->current_tvram].row[y].offset)
#define YPOS(y)      ((y) * 32 + 8)
#define R_XPOS(x, y) (((x) - lvns->tvram[lvns->current_tvram].row[y].offset)/24)
#define R_YPOS(y)    (((y) - 8) / 32)
#else
#define WIDTH2       320
#define CHARSIZE     12
#define CHARDRAWSIZE 14
#define EDYOFF       15
#define XPOS(x,y)    ((x) * 12 + lvns->tvram[lvns->current_tvram].row[y].offset)
#define YPOS(y)      ((y) * 14 + 4)
#define R_XPOS(x, y) (((x) - lvns->tvram[lvns->current_tvram].row[y].offset)/12)
#define R_YPOS(y)    (((y) - 4) / 14)
#endif

#ifdef USE_MGL
#define PATH_LEN 200

#ifndef MGLVNS_APPDIR
#define MGLVNS_APPDIR "/usr/local/lib/mglvns";
#endif

#define MGL_XOFFSET ((SCREEN_WIDTH - LVNS->width) / 2)
#define MGL_YOFFSET ((SCREEN_HEIGHT - LVNS->height) / 2)

/* δؿ¤Τ MGL2 ǤǺؤ */
#define LvnsImage mgImage
#define lvnsimage_new mglimage_new
#define lvnsimage_delete mglimage_delete
#define lvnsimage_load_lfg mglimage_load_lfg
#define lvnsimage_load_lf2 mglimage_load_lfg
#define lvnsSinEffect2 lvnsSinEffect
#define lvnsimage_add(s,d,x,y) mglimage_add(s,d,(x)/2,(y)/2)
#define lvnsimage_add2(s,d,x,y) mglimage_add(s,d,(x)/2,(y)/2)
#define lvnsimage_add3(s,d,x,y,p,t,i) mglimage_add3(s,d,(x)/2,(y)/2,p,t,i)
#define lvnsimage_add4(s,d,x,y,p) mglimage_add(s,d,(x)/2,(y)/2)
#define lvnsimage_circle(i,x,y,r,s,p) mglimage_circle(i,x,y,(r)/2,s,p)
/*
 * MGL2 Ǥξ X ǤȰ㤤ˤδؿƤӽФȡ
 */
#define lvnsimage_clear_index mglimage_clear_color
#endif

struct Lvns;

#include "LvnsMenu.h"

typedef enum {
	LVNS_JUMP_NONE,
	LVNS_JUMP_RESTART,
	LVNS_JUMP_TITLE,
	LVNS_JUMP_END
} LvnsJumpParameter;

typedef enum {
	LVNSTIMER_MAIN,
	LVNSTIMER_EFFECT,
	LVNSTIMER_SCRIPT,
} LvnsTimerType;

typedef enum {
    LVNS_CURSOR_PAGE,
    LVNS_CURSOR_KEY,
}LvnsCursorType;

typedef void (*LvnsBackEffectSetFunc)(struct Lvns *lvns);
typedef void (*LvnsBackEffectFunc)(struct Lvns *lvns);
typedef struct {
    LvnsBackEffectSetFunc set;
    LvnsBackEffectFunc func;
} LvnsBackEffectInfo;

typedef struct {
	int scn;  /* ʥꥪֹ */
	int blk;  /* ֥åֹ */
	int no;   /* ƥֹ */
} LvnsHistoryData;        /* ۥ⡼ */

typedef struct {
    char *command;
    int enable;
} LvnsCommandInfo;   /*  */

/**
 * ƥ¸
 */
typedef struct {

    int width;                   /* ΰΥ */
    int height;             
    int colors;                  /* ɬפʺ翧       */ 

    const char *leafpack_name;   /* ǡѥѥåե̾   */
    const char *scnpack_name;    /* ʥꥪѥѥåե̾ */
    const char *scn_name;        /* ʥꥪե١̾   */
    const char *fonttable_name;  /* ɥСȥơ֥̾ */

	MenuData *main_menu;         /* ᥤ˥塼 */

	void (*start)(struct Lvns *lvns);    /*        */
	void (*jingle)(struct Lvns *lvns);   /* 󥰥     */
	void (*opening)(struct Lvns *lvns);  /* ץ˥ */
	void (*title)(struct Lvns *lvns);    /* ȥ     */
	void (*main)(struct Lvns *lvns);     /* ᥤ       */

    void (*mergeCharacter)(struct Lvns *lvns);   /* 饯 */
    void (*clearScreen)(struct Lvns *lvns);      /*  VRAM ξõ */
    void (*loadBG)(struct Lvns *lvns, int no);   /* طɤ߹ */

    LvnsCommandInfo* (*getCommandList)(struct Lvns *lvns);           /*  */
    void (*execCommand)(struct Lvns *lvns, const char *command);     /*      */
    void (*drawChar)(struct Lvns *lvns, int x, int y, int, int);     /* ʸɽ     */

	void (*dispHistory)(struct Lvns *lvns, int pos);                 /* ɽ */

    int cursor_key;   /* ԤΥ */
    int cursor_page;  /* ڡԤΥ */

} LvnsSystemState;

typedef struct Lvns {

	/*  */
	enum {
		LVNS_JINGLE,
		LVNS_OP,
		LVNS_MAIN,
		LVNS_WAIT_MENU,
		LVNS_WAIT_KEY,
		LVNS_WAIT_PAGE,
		LVNS_WAIT_SELECT,
	} inside_state;

    /* ------------------------------------------------------------ */
	/* ǡѥ᡼ */

    LvnsSystemState *system_state;     /* ƥॹơ */

    char *data_path;        /* ǡܥѥ */
    char *savedata_path;    /* ǡ¸ */
 
    LeafPack *leafpack;     /* ǡѥѥåե */
    LeafPack *scnpack;      /* ʥꥪѥѥåե */
    u_char *leaf_font_data; /* Leaf FONT Υǡ   */

    u_char *leaf_to_euc;    /* ɥСȥơ֥ Leaf -> EUC  */
    u_short *jis_to_leaf;   /* ɥСȥơ֥ JIS Pack -> Leaf */

    /* ------------------------------------------------------ */
    /* ꥽ */

    Bool force_skip;      /* ʸǤ⤹äȤФ           */
    Bool key_click_fast;  /* åʸ®ɽ */
	Bool fast_when_seen;  /* ʸ᤯ɽ           */
    Bool enable_effect;       /* ̤Ȥ */
    Bool enable_effect_back;  /* طʸ̤Ȥ */

    Bool seen_mode;       /* ʸɤȤߤʤ⡼(for DEBUG) */
    Bool demo_mode;       /* ȥǥ⡼ */

    /* ------------------------------------------------------ */
    /* ƥѥ᡼ */

    Bool image_dark;    /* ߤΰŤ                      */

    int  char_wait_time;         /* ʸɽԤ     */
    
    Bool fast_text;    /* ƥȤФե饰       */
    Bool fast_disp;    /* ɽեȤäȤФե饰 */

	jmp_buf jump;      /*  */

    /* ------------------------------------------------------ */
    /* ߤξ ǡ¸ */
    enum bgtype {
        LVNS_VISUAL,
        LVNS_HCG,
        LVNS_BACKGROUND
    } bg_type;
    int bg_no;

	int character_no[3];

    Bool savepoint_flag;
    struct SavePoint{
        enum bgtype bg_type;
        int bg_no;
		int character_no[3];
        int scn;         /* ʥꥪֹ¸ */
        int blk;         /* ʥꥪ֥å¸       */
        long scn_offset; /* ʥꥪǡ¸ѥ */
        int current_music;
    } savepoint;
    struct SavePoint selectpoint;

    /* ---------------------------------------------------------- */
    /*  */

	/*  */
    Bool select;
	Bool cancel;
	Bool skip;
	Bool cursor_up;
	Bool cursor_down;
	Bool motion;
	int  motion_x;
	int  motion_y;
	int  number;

	Bool image_mode;
	
    /* ---------------------------------------------------------- */
    /* ʥꥪѡ */

    int start_scn_num;    /* ưʥꥪֹ(for DEBUG) */

    u_char *scn_data;     /* ʥꥪǡ             */
    u_char *scn_cur_head; /* ʥꥪǡƬ         */
    u_char *scn_cur;      /* ʥꥪǡ     */
    u_char *scn_text;     /* ʥꥪƥ           */

    size_t  scn_length;
	Bool    seen;         /* ɥʥꥪե饰         */

    int scn_current;      /* ߤΥʥꥪֹ         */
    int blk_current;      /* ߤΥʥꥪ֥å     */


    /* ------------------------------------------------------ */
    /* ƥȥ쥤 */

#define TEXT_WIDTH  26
#define TEXT_HEIGHT 16

    struct {
        /* ƥȥ쥤ѲVRAM */
		struct TextVramLine {
			int offset;
			struct TextVram {
				int code;       /* ʸΥ       */
				int attribute;  /* ʸ°         */
			} column[TEXT_WIDTH];
		} row[TEXT_HEIGHT];
        int cur_x;
        int cur_y;
        int o_cur_x;
        int o_cur_y;
    } tvram[2];
    int current_tvram;

    /* ξ */
    int cursor_state;
    
    /* ƥȥξ */
    int text_cursor_state;     

	/* ɽ° */
    int text_attr;

	/* ʸ񤭹ޤƤ뤫? */
	Bool text_written;

	/* ƥɽ⡼ɤɤ */
	Bool text_mode;

    /* ------------------------------------------------------- */
    /*  */

    LvnsImage *background; /* طʥǡݻ        */
    LvnsImage *vram;       /* طʹѲ VRAM */

    int latitude;          /* ߤβ뤵 0-255 */
    int latitude_dark;     /* ̤Ťε٤λ */

    int sepia_mode_next;   /* βڤؤǥԥĴɽ */
    int sepia_mode;        /* ԥĴɽ */

    LvnsBackEffectInfo *effect_back;
    LvnsBackEffectInfo *effect_back_next;
    int effect_back_state;                 /* طʥեѥơ */
#ifdef FONT_PLUS
    u_char*          font_file;
    LvnsFont	     *font;
#endif /* FONT_PLUS */

	void (*disp_func)(struct Lvns *lvns, void *data); /* ɲɽؿ */
	void *disp_func_param;

	int flush_x;
	int flush_y;
	int flush_w;
	int flush_h;

    /* -------------------------------------------------------- */
    /* ں */
	Music *music;

    /* 롼׽ */
    int current_music;                            /* ߱β     */
    Bool loop_music;                              /* 롼פɤ       */
    int next_music;                               /* Υ󤫤β */
    Bool loop_next_music;                         /* 롼פɤ       */

	/* ܥ塼 */
    int current_music_volume;                     /* ߤΥܥ塼 */
    int music_fade_mode;                          /* եɥ⡼       */
    int music_fade_flag;                          /* եɤ       */

    /* -------------------------------------------------------- */
    /* ɺ */
	Sound *sound;

    /* 롼׽ */
    int sound_loop;
    int sound_count;
    int interval_count;

    /* -------------------------------------------------------- */
	/* ҥȥ */
	LvnsHistoryData *history;
	int history_size;
	int history_pos;

#ifdef USE_MGL
    /* -------------------------------------------------------- */
    /* mglvns ΤߤǻȤ */
    int version;
#endif

    /* -------------------------------------------------------- */
	/* ƥ¸ */

    void *system_depend;                /* ưƥ¸ */

	void (*flushWindow)(struct Lvns *lvns);
	void (*flushWindowArea)(struct Lvns *lvns, int x, int y, int w, int h);
	void (*drawWindow)(struct Lvns *lvns);
	void (*drawWindowArea)(struct Lvns *lvns, int x, int y, int w, int h, int x2, int y2);
	void (*clearWindow)(struct Lvns *lvns);
	void (*clearWindowArea)(struct Lvns *lvns, int x, int y, int w, int h);
	void (*setPaletteIndex)(struct Lvns *lvns, int idx, int r, int g, int b);
	void (*setPalette)(struct Lvns *lvns);
	void (*setPaletteMulti)(struct Lvns *lvns, int par16);
	void (*setPaletteScreen)(struct Lvns *lvns, int par16);
	void (*drawWindowMask)(struct Lvns *lvns, int x, int y, int state);
	void (*drawWindowSquareMask)(struct Lvns *lvns, int x, int y, int state);
	void (*setDispOffset)(struct Lvns *lvns, int xoff, int yoff, int maxoff);
	void (*putPattern)(struct Lvns *lvns, int x, int y, int index, u_char *data);
	void (*putPattern2)(struct Lvns *lvns, int x, int y, int index, u_char *data);
	void (*resetTimer)(struct Lvns *lvns, int no);
	long (*getTimer)(struct Lvns *lvns, int no);
	void (*drawBox)(struct Lvns *lvns, int x, int y, int w, int h, int idx);
	time_t (*getFileTime)(struct Lvns *lvns, const char *path);
	void (*flip)(struct Lvns *lvns);

    /* -------------------------------------------------------- */


} Lvns;

/* LvnsDisp.c ɽ */
void LvnsDrawTextLayerAll(Lvns *lvns);
void LvnsDrawTextLayer(Lvns *lvns);
void LvnsClearCursor(Lvns *lvns);
void LvnsDrawCursor(Lvns *lvns, LvnsCursorType cursor_type);
void LvnsClearTextCursor(Lvns *lvns);
void LvnsDrawTextCursor(Lvns *lvns);
void LvnsDispWindow(Lvns *lvns);
void LvnsDispText(Lvns *lvns);
void LvnsTextOff(Lvns *lvns);
void LvnsTextOn(Lvns *lvns);

/* LvnsText.c ƥȽ */
void LvnsLocate(Lvns *lvns, int x, int y);
void LvnsPutChar(Lvns *lvns, int c, int attr);
void LvnsPutCharNormal(Lvns *lvns, int c, int attr);
void LvnsPuts(Lvns *lvns, const u_char *str, int attr);
void LvnsClearText(Lvns *lvns);
void LvnsSetTextOffset(Lvns *lvns, int offset);
void LvnsNewLineText(Lvns *lvns);
void LvnsDrawString(Lvns *lvns, int x, int y, const u_char *str, int attr);
void LvnsDrawStringCenter(Lvns *lvns, int y, const u_char *str, int attr);

/* LvnsMusic.c  */
void LvnsSetMusicVolume(Lvns *lvns, int no);
void LvnsStartMusic(Lvns *lvns, int no);
void LvnsStopMusic(Lvns *lvns);
void LvnsPauseMusic(Lvns *lvns);
void LvnsStartMusicLoop(Lvns *lvns, int no);
void LvnsStartMusicLoop2(Lvns *lvns, int no);
void LvnsSetNextMusic(Lvns *lvns, int no);
void LvnsSetNextMusicLoop(Lvns *lvns, int no);
void LvnsFadeMusic(Lvns *lvns);
void LvnsStartNextMusic(Lvns *lvns);
void LvnsLoopMusic(Lvns *lvns);
void LvnsWaitMusic(Lvns *lvns);
void LvnsWaitMusicFade(Lvns *lvns);

/* LvnsSound.c ̲ */
void LvnsLoadSound(Lvns *lvns, const char* basename, int no);
void LvnsLoadSound2(Lvns *lvns, const char* name);
void LvnsStartSound(Lvns *lvns, int count);
void LvnsStopSound(Lvns *lvns);
void LvnsWaitSound(Lvns *lvns);
void LvnsLoopSound(Lvns *lvns);

/* LvnsBackEffect.c طʥե */
void LvnsSetBackEffect(Lvns *lvns, LvnsBackEffectInfo *info);
void LvnsSetNextBackEffect(Lvns *lvns, LvnsBackEffectInfo *info);
void LvnsBackEffectSetState(Lvns *lvns);
void LvnsBackEffect(Lvns *lvns);

/* LvnsFile.c եɤ߹ */
u_char *LvnsLoadData(Lvns *lvns, const char *name, size_t *size);
LvnsImage *LvnsLoadImage(Lvns *lvns, const char *name, LvnsImage *over);
void LvnsLoadScenario(Lvns *lvns, int scn, int blk);
void LvnsLoadScenarioBlock(Lvns *lvns, int blk);
u_char *LvnsGetScenarioText(Lvns *lvns, int no);
void LvnsLoadBackground(Lvns *lvns, const char *basename, int no);

/* LvnsEtc.c */
void LvnsInitSavePoint(Lvns *lvns, struct SavePoint *sp);
void LvnsSetSavePoint(Lvns *lvns, struct SavePoint *sp);

/* LvnsControl.c */
void LvnsMotion(Lvns *lvns, int x, int y);
void LvnsSelect(Lvns *lvns);
void LvnsCancel(Lvns *lvns);
void LvnsSetImageMode(Lvns *lvns);
void LvnsSkipTillSelect(Lvns *lvns);
void LvnsCursorUp(Lvns *lvns);
void LvnsCursorDown(Lvns *lvns);
void LvnsInputNumber(Lvns *lvns, int no);
void LvnsImageMode(Lvns *lvns);
void LvnsWaitKey(Lvns *lvns);
void LvnsWaitPage(Lvns *lvns);
int  LvnsWaitSelect(Lvns *lvns, int select_num);
void LvnsWaitClick(Lvns *lvns);

/* Lvns.c */
Lvns* LvnsNew(void);
void  LvnsInitialize(Lvns *lvns, int type, void *system_depend);
void  LvnsDestroy(Lvns *lvns);
void  LvnsFlip(Lvns *lvns, Bool disp_ok);
void  LvnsMain(Lvns *lvns);
void  LvnsWait(Lvns *lvns, int flip_cnt);
void  LvnsJump(Lvns *lvns, LvnsJumpParameter param);
void  LvnsSetDataPath(Lvns *lvns, const char *path);
void  LvnsSetSaveDataPath(Lvns *lvns, const char *path);
void  LvnsSetDispFunc(Lvns *lvns, void (*func)(Lvns *lvns, void *data), void *data);

void LvnsFlushWindow(Lvns *lvns);
void LvnsFlushWindowArea(Lvns *lvns, int x, int y, int w, int h);

/* LvnsMenu.c */
void LvnsDrawMenu(Lvns *lvns);

/* LvnsHistory.c */
void LvnsAddHistory(Lvns *lvns, int no);
void LvnsHistoryMode(Lvns *lvns);

#ifdef USE_MGL
/* mgMain.c */
void mgLvnsQuit(Lvns *);

/* mgLvns.c */
void mgLvnsInit(Lvns *);
void mgLvnsDelete(Lvns *);
void mgLvnsStart(Lvns *);

/* mgLvnsCore.c */
void mgLvnsCoreSetLvns(Lvns *);

/* mgAction.c */
void MglGetEvent(Lvns *, int);

/* sizuku.c */
void SizukuInitialize(Lvns *);

/* kizuato.c */
void KizuatoInitialize(Lvns *);

/* toheart.c */
void ToHeartInitialize(Lvns *);
#endif

#endif
